<?php

namespace Drupal\charting\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;

/**
 * Chart style plugin.
 *
 * @ViewsStyle(
 *   id = "charting_chart",
 *   title = @Translation("Chart"),
 *   help = @Translation("Render charts."),
 *   theme = "views_style_charting_chart",
 *   display_types = {"normal"}
 * )
 */
class Chart extends StylePluginBase {

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowClass = TRUE;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['wrapper_class'] = ['default' => 'item-list'];
    $options['title_field'] = ['default' => ''];
    $options['value_field'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    // Build select options.
    $labels = $this->displayHandler->getFieldLabels();
    $title_options = [];
    $value_options = [];
    $fields = $this->displayHandler->getOption('fields');
    foreach ($fields as $field_name => $field) {
      $title_options[$field_name] = $labels[$field_name];
      $value_options[$field_name] = $labels[$field_name];
    }

    // Build form.
    $form['wrapper_class'] = [
      '#title' => $this->t('Wrapper class'),
      '#description' => $this->t('The class to provide on the wrapper, outside rows.'),
      '#type' => 'textfield',
      '#default_value' => $this->options['wrapper_class'],
    ];

    $form['title_field'] = [
      '#title' => $this->t('Title source field'),
      '#type' => 'select',
      '#default_value' => $this->options['title_field'],
      '#description' => $this->t("The source of the title for each item."),
      '#options' => $title_options,
      '#empty_value' => 'none',
    ];

    $form['value_field'] = [
      '#title' => $this->t('Value source field'),
      '#type' => 'select',
      '#default_value' => $this->options['value_field'],
      '#description' => $this->t("The source of the value for each item."),
      '#options' => $value_options,
      '#empty_value' => 'none',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    if (!empty($this->options['title_field']) && $this->options['title_field'] != 'none') {
      $title_field = $this->options['title_field'];
    }

    if (!empty($this->options['value_field']) && $this->options['value_field'] != 'none') {
      $value_field = $this->options['value_field'];
    }

    $dom_id = $this->view->dom_id;

    $build = [
      '#theme' => $this->view->buildThemeFunctions('views_style_charting_chart'),
      '#view' => $this->view,
      '#id' => $dom_id,
      '#attached' => [
        'library' => [
          'charting/charting_chart',
        ],
      ],
    ];

    // Add cache dependency for the view.
    $renderer = $this->getRenderer();
    $renderer->addCacheableDependency($build, $this->view->storage);

    $this->renderFields($this->view->result);

    /*
     * Add points to output.
     */
    foreach ($this->view->result as $row_number => $row) {
      if (!empty($title_field)) {
        if (!empty($this->rendered_fields[$row_number][$title_field])) {
          $title_build = $this->rendered_fields[$row_number][$title_field];
        }
        elseif (!empty($this->view->field[$title_field])) {
          $title_build = $this->view->field[$title_field]->render($row);
        }
      }

      if (!empty($value_field)) {
        $value_build = $this->view->field[$value_field]->render($row);
      }

      $row = [
        'serial' => $row_number,
        'content' => $this->view->rowPlugin->render($row),
        'title' => empty($title_build) ? '' : $title_build,
        'value' => empty($value_build) ? 0 : $value_build,
      ];

      $build['#rows'][] = $row;
    }

    return $build;
  }

}
